import 'dart:math' as math;

import 'package:knt_bloc/knt_bloc.dart';

import '../model/photo.dart';

class PagingGalleryBloc extends BasePagingBloc<String?, Photo> {
  static final PagingGalleryBloc _singleton = PagingGalleryBloc._internal();

  factory PagingGalleryBloc() {
    return _singleton;
  }

  PagingGalleryBloc._internal() {
    _photos = [];
  }

  late final List<Photo> _photos;

  @override
  Stream<BaseState> handleEvent(BaseEvent event) async* {
    if (event is LoadGalleryPhotosEvent) {
      yield* _handleLoadGalleryPhotosEvent(event);
      return;
    }
    yield* super.handleEvent(event);
  }

  Stream<BaseState> _handleLoadGalleryPhotosEvent(
      LoadGalleryPhotosEvent event) async* {
    _photos.clear();
    _photos.addAll(event.photos);
  }

  @override
  Future<BaseResponsePaging<Photo>> pagingInProgress(
      BaseRequestPaging<String?> pagingQuery) async {
    List<Photo> photos = _photos.sublist(
      pagingQuery.offset,
      math.min(pagingQuery.offset + pagingQuery.limit, _photos.length),
    );
    return BaseResponsePaging<Photo>(dataList: photos);
  }

  @override
  BasePagingResultState<Photo> pagingStateInstance(
    List<Photo> dataList,
    int offset, {
    String? tag,
  }) =>
      FetchedGalleryPhotosState(dataList, offset, tag: tag);
}

// region EVENT
class LoadGalleryPhotosEvent extends BaseEvent {
  final List<Photo> photos;

  const LoadGalleryPhotosEvent(this.photos);
}
// endregion

// region STATE
class FetchedGalleryPhotosState extends BasePagingResultState<Photo> {
  const FetchedGalleryPhotosState(
    List<Photo> dataList,
    int currentOffset, {
    String? tag,
  }) : super(dataList, currentOffset, tag: tag);
}
// endregion
